# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .table_extractor import TableExtractor


class ListTableExtractorsResponse(UniversalBaseModel):
    """
    Response containing a list of available table extractors for document processing
    """

    table_extractors: typing.Optional[typing.List[TableExtractor]] = pydantic.Field(default=None)
    """
    An array of table extractors.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
