# This file was auto-generated by Fern from our API Definition.

import typing

QueryWarning = typing.Union[
    typing.Literal["exceeded_max_input_length_fcs", "intelligent_query_rewriting_failed"], typing.Any
]
