# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from .corpus_key import CorpusKey
from .search_corpus import SearchCorpus


class KeyedSearchCorpus(SearchCorpus):
    """
    A corpus with its identifying key for use in search operations within a customer account.
    """

    corpus_key: typing.Optional[CorpusKey] = None
    query: typing.Optional[str] = pydantic.Field(default=None)
    """
    Query for a particular corpus that will override the overall query if present.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
