# This file was auto-generated by Fern from our API Definition.

import typing

from .generation_info import GenerationInfo
from .stream_error import StreamError
from .stream_generation_chunk import StreamGenerationChunk
from .stream_generation_end import StreamGenerationEnd
from .stream_response_end import StreamResponseEnd

SummarizeDocumentStreamedResponse = typing.Union[
    StreamGenerationChunk, GenerationInfo, StreamGenerationEnd, StreamError, StreamResponseEnd
]
