# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class ExtractionUsage(UniversalBaseModel):
    """
    How much extraction quota the document used. This information is currently not returned when
    retrieving the document, and only returned when indexing a document.
    """

    table_extraction_used: typing.Optional[int] = pydantic.Field(default=None)
    """
    The number of pages from the document that consumed the extraction quota.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
