# This file was auto-generated by Fern from our API Definition.

import typing

from .factual_consistency_score_span import FactualConsistencyScoreSpan
from .generation_span import GenerationSpan
from .rephrase_span import RephraseSpan
from .rerank_span import RerankSpan
from .rewritten_query_span import RewrittenQuerySpan
from .search_span import SearchSpan

QueryHistorySpan = typing.Union[
    RephraseSpan, SearchSpan, RerankSpan, GenerationSpan, FactualConsistencyScoreSpan, RewrittenQuerySpan
]
