# This file was auto-generated by Fern from our API Definition.

from .create_open_ai_encoder_request import CreateOpenAiEncoderRequest

ComponentsSchemasCreateOpenAiEncoderRequest = CreateOpenAiEncoderRequest
