# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .generation_preset import GenerationPreset
from .list_metadata import ListMetadata


class ListGenerationPresetsResponse(UniversalBaseModel):
    """
    Response containing a list of generation presets for configuring RAG behavior.
    """

    generation_presets: typing.Optional[typing.List[GenerationPreset]] = pydantic.Field(default=None)
    """
    List of generation presets.
    """

    metadata: typing.Optional[ListMetadata] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
