# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class UsersResetPasswordResponse(UniversalBaseModel):
    one_time_code: typing.Optional[str] = pydantic.Field(default=None)
    """
    A one time code that can be used to reset the password. This is optional, and is only populated for some Vectara deployments.
    """

    one_time_code_link: typing.Optional[str] = pydantic.Field(default=None)
    """
    The link where to enter the `one_time_code`.
    This is optional, and is only populated for some Vectara deployments.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
