# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .evaluate_factual_consistency_request_model_parameters import EvaluateFactualConsistencyRequestModelParameters

__all__ = ["EvaluateFactualConsistencyRequestModelParameters"]
